#include "source.hpp"
#include "dbcore.hpp"
#include "exception.hpp"
#include "library.hpp"
#include <QDir>
#include <QDirIterator>

using namespace Library;

QMap<int, Source*>	Source::sourcesMap;

Source::Source(Type type, const QString &path, bool recursive, const QString &host,
			   unsigned short port, const QString &username,
			   const QString &password, const QString &filename) :
	type(type), port(port), host(host), path(path),
	username(username), password(password), filename(filename), recursive(recursive)
{
	updatedAt = QDateTime::currentDateTime();
	status = UNINITIALIZED;
	id = 0;
	save();
}

Source::Source(int id)
{
	QSqlQuery query;
	query.prepare("SELECT type, path, host, port, username, password, filename, updatedAt, status, recursive "
				  "FROM sources WHERE id = :id LIMIT 1");
	query.bindValue(":id", id);
	query.exec();
	if (query.first())
	{
		this->id = id;
		type = static_cast<Type>(query.value(0).toInt());
		path = query.value(1).toString();
		host = query.value(2).toString();
		port = query.value(3).toUInt();
		username = query.value(4).toString();
		password = query.value(5).toString();
		filename = query.value(6).toString();
		updatedAt = query.value(7).toDateTime();
		status = static_cast<Status>(query.value(8).toInt());
		recursive = query.value(9).toBool();
	}
	else
		qDebug() << QString("Can't load source with id %1: %2 (%3)").arg(id).arg(query.lastError().text()).arg(query.lastQuery());
}


Source*	Source::getSource(int id)
{
	if (!sourcesMap.contains(id))
		sourcesMap[id] = new Source(id);
	return sourcesMap[id];
}

Source::Status Source::update()
{
	QMap<QString, int>	collection;
	qDebug() << "Update source" << toUrl().toString();
	{
		QDirIterator filesList(toUrl().toLocalFile(), QDirIterator::Subdirectories | QDirIterator::FollowSymlinks);
		QSqlQuery query;
		query.prepare("SELECT id, path FROM audio_tracks WHERE source = :source");
		query.bindValue(":source", id);
		query.exec();
		while(query.next())
		{
			int		id = query.value(0).toInt();
			QString	path = query.value(1).toString();
			collection[path] = id;
		}
		int i = 0;
		while (filesList.hasNext())
		{
			filesList.next();
			++i;
		}
		emit addToMaximumProgress(i);
	}
	{
		QDirIterator filesList(toUrl().toLocalFile(), QDirIterator::Subdirectories | QDirIterator::FollowSymlinks);
		int i = 0;
		while (filesList.hasNext())
		{
			i++;
			if (i % 10 == 0)
				emit progressValueChange(10);
			filesList.next();
			if (!filesList.fileInfo().isFile())
				continue;
			if (LibraryPlugin::allowedExtension.contains(filesList.fileInfo().suffix(), Qt::CaseInsensitive))
			{
				QString	relativePath = filesList.fileInfo().filePath();
				relativePath.remove(QRegExp("^"+ QRegExp::escape(toUrl().toLocalFile())));
				if (collection.contains(relativePath))
				{
					collection.remove(relativePath);
				}
				else
				{
					AudioTrack	track(relativePath, this);
					track.save();
				}
			}
		}
	}

	foreach (int i, collection.values())
		AudioTrack::getTrack(i)->remove();
	return (status = OK);
}


bool	Source::newRecord() const
{
	return id == 0;
}

void	Source::save()
{
	QSqlQuery query;

	if (newRecord())
		query.prepare("INSERT INTO sources(type, path, host, port, username, password, "
					  "filename, updatedAt, status, recursive) VALUES (:type, :path, :host, :port, "
					  ":username, :password, :filename, :updatedAt, :status, :recursive)");
	else
		query.prepare("UPDATE sources SET type = :type, path = :path, host = :host"
					  "port= :port, username = :username, password = :password, filename = :filename, "
					  "updatedAt = :updatedAt, status = :status, recursive = :recursive "
					  "WHERE id = :id");
	query.bindValue(":type", type);
	query.bindValue(":path", path);
	query.bindValue(":host", host);
	query.bindValue(":port", port);
	query.bindValue(":username", username);
	query.bindValue(":password", password);
	query.bindValue(":filename", filename);
	query.bindValue(":updatedAt", updatedAt);
	query.bindValue(":status", status);
	query.bindValue(":recursive", recursive);

	if (!query.exec())
		qDebug() << QString("Can't save source: %1").arg(query.lastError().text());
	if (newRecord())
	{
		id = query.lastInsertId().toInt();
		sourcesMap[id] = this;
	}
}

const QString& Source::getPath() const
{
	return path;
}

void Source::merge(Source *oldSource)
{
	QString newPrefix = oldSource->getPath();
	newPrefix.remove(QRegExp("^" + QRegExp::escape(path)));
	QSqlQuery query;
	query.prepare("UPDATE audio_tracks SET path = :prefix || path, source = :source_id WHERE source = :old_source_id;");
	query.bindValue(":prefix", newPrefix);
	query.bindValue(":source_id", getId());
	query.bindValue(":old_source_id", oldSource->getId());
	if (!query.exec())
		throw_exception(0x01, tr("Can't merge source: %1").arg(query.lastError().text()));
	const QMap<int, AudioTrack*>& tracks = AudioTrack::getTracksMap();
	QMap<int, AudioTrack*>::const_iterator it = tracks.constBegin();
	while (it != tracks.constEnd())
	{
		AudioTrack *track = it.value();
		if (track->getSource() == oldSource)
		{
			track->setSource(this);
			track->setPath(QString(newPrefix + track->getPath()));
		}
		++it;
	}

	//Check username/password modification
}

void Source::remove()
{
	QSqlQuery query;
	query.prepare("DELETE FROM sources WHERE id = :id");
	query.bindValue(":id", getId());
	if (!query.exec())
		throw_exception(0x02, tr("Cannot remove source: %1").arg(query.lastError().text()));
	sourcesMap.remove(getId());
	delete this;
}

QUrl Source::toUrl() const
{
	QString protocol;
	QUrl url;

	if (type == HTTP)
		protocol = "http://";
	else if (type == FTP)
		protocol = "ftp://";

	if (port)
		url = QUrl::fromUserInput(protocol + host + ":" + QVariant(port).toString() + path + filename);
	else
		url = QUrl::fromUserInput(protocol + host + path + filename);
	if (!username.isEmpty())
		url.setUserName(username);
	if (!password.isEmpty())
		url.setPassword(password);
	return url;
}

bool Source::isRecursive() const
{
	return recursive;
}

int	Source::getId() const
{
	return id;
}

Source::Type Source::typeFromScheme(const QString &scheme)
{
	if (scheme == "http")
		return HTTP;
	else if (scheme == "ftp")
		return FTP;
	else
		return LOCAL;
}
